@@ -53,6 +53,7 @@
 #include "qgschunknode.h"
 #include "qgseventtracing.h"
 #include "qgsgeotransform.h"
+#include "qgsglobechunkedentity.h"
 #include "qgsmaterial.h"
 #include "qgsmeshlayer.h"
 #include "qgsmeshlayer3drenderer.h"
@@ -71,6 +72,7 @@
 #include "qgspoint3dbillboardmaterial.h"
 #include "qgsmaplayertemporalproperties.h"
 #include "qgsmaplayerelevationproperties.h"
+
 #include "qgslinematerial_p.h"
 #include "qgs3dsceneexporter.h"
 #include "qgs3dmapexportsettings.h"
@@ -83,6 +85,9 @@
 
 #include "qgswindow3dengine.h"
 #include "qgspointcloudlayer.h"
+#include "qgsforwardrenderview.h"
+#include "qgsambientocclusionrenderview.h"
+#include "qgspostprocessingentity.h"
 
 std::function<QMap<QString, Qgs3DMapScene *>()> Qgs3DMapScene::sOpenScenesFunction = [] { return QMap<QString, Qgs3DMapScene *>(); };
 
@@ -113,7 +118,11 @@ Qgs3DMapScene::Qgs3DMapScene( Qgs3DMapSettings &map, QgsAbstract3DEngine *engine
 
   // Camera controlling
   mCameraController = new QgsCameraController( this ); // attaches to the scene
-  mCameraController->resetView( 1000 );
+
+  if ( mMap.sceneMode() == Qgis::SceneMode::Globe )
+    mCameraController->resetGlobe( 10'000'000 );
+  else
+    mCameraController->resetView( 1000 );
 
   addCameraViewCenterEntity( mEngine->camera() );
   addCameraRotationCenterEntity( mCameraController );
@@ -202,7 +211,6 @@ Qgs3DMapScene::Qgs3DMapScene( Qgs3DMapSettings &map, QgsAbstract3DEngine *engine
   // force initial update of ambient occlusion settings
   onAmbientOcclusionSettingsChanged();
 
-  mCameraController->setCameraNavigationMode( mMap.cameraNavigationMode() );
   onCameraMovementSpeedChanged();
 
   on3DAxisSettingsChanged();
@@ -210,36 +218,46 @@ Qgs3DMapScene::Qgs3DMapScene( Qgs3DMapSettings &map, QgsAbstract3DEngine *engine
 
 void Qgs3DMapScene::viewZoomFull()
 {
+  if ( mMap.sceneMode() == Qgis::SceneMode::Globe )
+  {
+    mCameraController->resetGlobe( 10'000'000 );
+    return;
+  }
+
   const QgsDoubleRange zRange = elevationRange();
   const QgsRectangle extent = sceneExtent();
   const double side = std::max( extent.width(), extent.height() );
   double d = side / 2 / std::tan( cameraController()->camera()->fieldOfView() / 2 * M_PI / 180 );
   d += zRange.isInfinite() ? 0. : zRange.upper();
   mCameraController->resetView( static_cast<float>( d ) );
-  return;
 }
 
 void Qgs3DMapScene::setViewFrom2DExtent( const QgsRectangle &extent )
 {
   QgsPointXY center = extent.center();
-  QgsVector3D centerWorld = mMap.mapToWorldCoordinates( QVector3D( center.x(), center.y(), 0 ) );
-  QgsVector3D p1 = mMap.mapToWorldCoordinates( QVector3D( extent.xMinimum(), extent.yMinimum(), 0 ) );
-  QgsVector3D p2 = mMap.mapToWorldCoordinates( QVector3D( extent.xMaximum(), extent.yMaximum(), 0 ) );
+  const QgsVector3D origin = mMap.origin();
 
-  float xSide = std::abs( p1.x() - p2.x() );
-  float ySide = std::abs( p1.z() - p2.z() );
-  if ( xSide < ySide )
-  {
-    float fov = 2 * std::atan( std::tan( qDegreesToRadians( cameraController()->camera()->fieldOfView() ) / 2 ) * cameraController()->camera()->aspectRatio() );
-    float r = xSide / 2.0f / std::tan( fov / 2.0f );
-    mCameraController->setViewFromTop( centerWorld.x(), centerWorld.z(), r );
-  }
-  else
-  {
-    float fov = qDegreesToRadians( cameraController()->camera()->fieldOfView() );
-    float r = ySide / 2.0f / std::tan( fov / 2.0f );
-    mCameraController->setViewFromTop( centerWorld.x(), centerWorld.z(), r );
-  }
+  const QgsVector3D p1 = mMap.mapToWorldCoordinates( QgsVector3D( extent.xMinimum(), extent.yMinimum(), 0 ) );
+  const QgsVector3D p2 = mMap.mapToWorldCoordinates( QgsVector3D( extent.xMaximum(), extent.yMaximum(), 0 ) );
+
+  const double xSide = std::abs( p1.x() - p2.x() );
+  const double ySide = std::abs( p1.y() - p2.y() );
+  const double side = std::max( xSide, ySide );
+
+  const double fov = qDegreesToRadians( cameraController()->camera()->fieldOfView() );
+  double distance = side / 2.0f / std::tan( fov / 2.0f );
+
+  // adjust by elevation
+  const QgsDoubleRange zRange = elevationRange();
+  if ( !zRange.isInfinite() )
+    distance += zRange.upper();
+
+  // subtract map origin so coordinates are relative to it
+  mCameraController->setViewFromTop(
+    static_cast<float>( center.x() - origin.x() ),
+    static_cast<float>( center.y() - origin.y() ),
+    static_cast<float>( distance )
+  );
 }
 
 QVector<QgsPointXY> Qgs3DMapScene::viewFrustum2DExtent() const
@@ -272,11 +290,6 @@ QVector<QgsPointXY> Qgs3DMapScene::viewFrustum2DExtent() const
   return extent;
 }
 
-int Qgs3DMapScene::terrainPendingJobsCount() const
-{
-  return mTerrain ? mTerrain->pendingJobsCount() : 0;
-}
-
 int Qgs3DMapScene::totalPendingJobsCount() const
 {
   int count = 0;
@@ -310,20 +323,26 @@ void Qgs3DMapScene::onCameraChanged()
   }
 
   updateScene( true );
-  bool changedCameraPlanes = updateCameraNearFarPlanes();
-
-  if ( changedCameraPlanes )
-  {
-    // repeat update of entities - because we have updated camera's near/far planes,
-    // the active nodes may have changed as well
-    updateScene( true );
-    updateCameraNearFarPlanes();
-  }
+  updateCameraNearFarPlanes();
 
   onShadowSettingsChanged();
 
   QVector<QgsPointXY> extent2D = viewFrustum2DExtent();
   emit viewed2DExtentFrom3DChanged( extent2D );
+
+  // The magic to make things work better in large scenes (e.g. more than 50km across)
+  // is here: we will simply move the origin of the scene, and update transforms
+  // of the camera and all other entities. That should ensure we will not need to deal
+  // with large coordinates in 32-bit floats (and if we do have large coordinates,
+  // because the scene is far from the camera, we don't care, because those errors
+  // end up being tiny when viewed from far away).
+  constexpr float ORIGIN_SHIFT_THRESHOLD = 10'000;
+  if ( mSceneOriginShiftEnabled && mEngine->camera()->position().length() > ORIGIN_SHIFT_THRESHOLD )
+  {
+    const QgsVector3D newOrigin = mMap.origin() + QgsVector3D( mEngine->camera()->position() );
+    QgsDebugMsgLevel( QStringLiteral( "Rebasing scene origin from %1 to %2" ).arg( mMap.origin().toString( 1 ), newOrigin.toString( 1 ) ), 2 );
+    mMap.setOrigin( newOrigin );
+  }
 }
 
 void Qgs3DMapScene::updateScene( bool forceUpdate )
@@ -334,8 +353,7 @@ void Qgs3DMapScene::updateScene( bool forceUpdate )
     return;
   }
 
-  if ( forceUpdate )
-    QgsEventTracing::addEvent( QgsEventTracing::Instant, QStringLiteral( "3D" ), QStringLiteral( "Update Scene" ) );
+  QgsEventTracing::ScopedEvent traceEvent( QStringLiteral( "3D" ), forceUpdate ? QStringLiteral( "Force update scene" ) : QStringLiteral( "Update scene" ) );
 
   Qgs3DMapSceneEntity::SceneContext sceneContext;
   Qt3DRender::QCamera *camera = mEngine->camera();
@@ -343,7 +361,22 @@ void Qgs3DMapScene::updateScene( bool forceUpdate )
   sceneContext.cameraPos = camera->position();
   const QSize size = mEngine->size();
   sceneContext.screenSizePx = std::max( size.width(), size.height() ); // TODO: is this correct?
-  sceneContext.viewProjectionMatrix = camera->projectionMatrix() * camera->viewMatrix();
+
+  // Make our own projection matrix so that frustum culling done by the
+  // entities isn't dependent on the current near/far planes, which would then
+  // require multiple steps to stabilize.
+  // The matrix is constructed just like in QMatrix4x4::perspective(), but for
+  // all elements involving the near and far plane, the limit of the expression
+  // with the far plane going to infinity is taken.
+  float fovRadians = ( camera->fieldOfView() / 2.0f ) * static_cast<float>( M_PI ) / 180.0f;
+  float fovCotan = std::cos( fovRadians ) / std::sin( fovRadians );
+  QMatrix4x4 projMatrix(
+    fovCotan / camera->aspectRatio(), 0, 0, 0,
+    0, fovCotan, 0, 0,
+    0, 0, -1, -2,
+    0, 0, -1, 0
+  );
+  sceneContext.viewProjectionMatrix = projMatrix * camera->viewMatrix();
 
 
   for ( Qgs3DMapSceneEntity *entity : std::as_const( mSceneEntities ) )
@@ -421,6 +454,8 @@ bool Qgs3DMapScene::updateCameraNearFarPlanes()
 
 void Qgs3DMapScene::onFrameTriggered( float dt )
 {
+  QgsEventTracing::addEvent( QgsEventTracing::EventType::Instant, QStringLiteral( "3D" ), QStringLiteral( "Frame begins" ) );
+
   mCameraController->frameTriggered( dt );
 
   updateScene();
@@ -457,6 +492,14 @@ void Qgs3DMapScene::createTerrain()
     mTerrain = nullptr;
   }
 
+  if ( mGlobe )
+  {
+    mSceneEntities.removeOne( mGlobe );
+
+    delete mGlobe;
+    mGlobe = nullptr;
+  }
+
   if ( !mTerrainUpdateScheduled )
   {
     // defer re-creation of terrain: there may be multiple invocations of this slot, so create the new entity just once
@@ -472,7 +515,14 @@ void Qgs3DMapScene::createTerrain()
 
 void Qgs3DMapScene::createTerrainDeferred()
 {
-  if ( mMap.terrainRenderingEnabled() && mMap.terrainGenerator() )
+  QgsChunkedEntity *terrainOrGlobe = nullptr;
+
+  if ( mMap.sceneMode() == Qgis::SceneMode::Globe && mMap.terrainRenderingEnabled() )
+  {
+    mGlobe = new QgsGlobeEntity( &mMap );
+    terrainOrGlobe = mGlobe;
+  }
+  else if ( mMap.sceneMode() == Qgis::SceneMode::Local && mMap.terrainRenderingEnabled() && mMap.terrainGenerator() )
   {
     double tile0width = mMap.terrainGenerator()->rootChunkExtent().width();
     int maxZoomLevel = Qgs3DUtils::maxZoomLevel( tile0width, mMap.terrainSettings()->mapTileResolution(), mMap.terrainSettings()->maximumGroundError() );
@@ -482,14 +532,18 @@ void Qgs3DMapScene::createTerrainDeferred()
     mMap.terrainGenerator()->setupQuadtree( rootBox3D, rootError, maxZoomLevel, clippingBox3D );
 
     mTerrain = new QgsTerrainEntity( &mMap );
-    mTerrain->setParent( this );
-    mTerrain->setShowBoundingBoxes( mMap.showTerrainBoundingBoxes() );
+    terrainOrGlobe = mTerrain;
+  }
 
-    mSceneEntities << mTerrain;
+  if ( terrainOrGlobe )
+  {
+    terrainOrGlobe->setParent( this );
+    terrainOrGlobe->setShowBoundingBoxes( mMap.showTerrainBoundingBoxes() );
 
-    connect( mTerrain, &QgsChunkedEntity::pendingJobsCountChanged, this, &Qgs3DMapScene::totalPendingJobsCountChanged );
-    connect( mTerrain, &QgsTerrainEntity::pendingJobsCountChanged, this, &Qgs3DMapScene::terrainPendingJobsCountChanged );
-    connect( mTerrain, &Qgs3DMapSceneEntity::newEntityCreated, this, [this]( Qt3DCore::QEntity *entity ) {
+    mSceneEntities << terrainOrGlobe;
+
+    connect( terrainOrGlobe, &QgsChunkedEntity::pendingJobsCountChanged, this, &Qgs3DMapScene::totalPendingJobsCountChanged );
+    connect( terrainOrGlobe, &Qgs3DMapSceneEntity::newEntityCreated, this, [this]( Qt3DCore::QEntity *entity ) {
       // let's make sure that any entity we're about to show has the right scene origin set
       const QList<QgsGeoTransform *> transforms = entity->findChildren<QgsGeoTransform *>();
       for ( QgsGeoTransform *transform : transforms )
@@ -501,10 +555,6 @@ void Qgs3DMapScene::createTerrainDeferred()
       handleClippingOnEntity( entity );
     } );
   }
-  else
-  {
-    mTerrain = nullptr;
-  }
 
   // make sure that renderers for layers are re-created as well
   const QList<QgsMapLayer *> layers = mMap.layers();
@@ -606,9 +656,39 @@ void Qgs3DMapScene::updateTemporal()
   }
 }
 
+void Qgs3DMapScene::addSceneEntity( Qgs3DMapSceneEntity *sceneNewEntity )
+{
+  Q_ASSERT( sceneNewEntity );
+
+  mSceneEntities.append( sceneNewEntity );
+
+  sceneNewEntity->setParent( this );
+
+  finalizeNewEntity( sceneNewEntity );
+
+  connect( sceneNewEntity, &Qgs3DMapSceneEntity::newEntityCreated, this, [this]( Qt3DCore::QEntity *entity ) {
+    finalizeNewEntity( entity );
+    // this ensures to update the near/far planes with the exact bounding box of the new entity.
+    updateCameraNearFarPlanes();
+  } );
+
+  connect( sceneNewEntity, &Qgs3DMapSceneEntity::pendingJobsCountChanged, this, &Qgs3DMapScene::totalPendingJobsCountChanged );
+
+  onCameraChanged(); // needed for chunked entities
+}
+
+void Qgs3DMapScene::removeSceneEntity( Qgs3DMapSceneEntity *sceneEntity )
+{
+  Q_ASSERT( sceneEntity );
+
+  mSceneEntities.removeOne( sceneEntity );
+
+  sceneEntity->deleteLater();
+}
+
+
 void Qgs3DMapScene::addLayerEntity( QgsMapLayer *layer )
 {
-  bool needsSceneUpdate = false;
   QgsAbstract3DRenderer *renderer = layer->renderer3D();
   if ( renderer )
   {
@@ -670,30 +750,21 @@ void Qgs3DMapScene::addLayerEntity( QgsMapLayer *layer )
     Qt3DCore::QEntity *newEntity = renderer->createEntity( &mMap );
     if ( newEntity )
     {
-      newEntity->setParent( this );
       mLayerEntities.insert( layer, newEntity );
 
-      finalizeNewEntity( newEntity );
-
       if ( Qgs3DMapSceneEntity *sceneNewEntity = qobject_cast<Qgs3DMapSceneEntity *>( newEntity ) )
       {
-        needsSceneUpdate = true;
-        mSceneEntities.append( sceneNewEntity );
-
-        connect( sceneNewEntity, &Qgs3DMapSceneEntity::newEntityCreated, this, [this]( Qt3DCore::QEntity *entity ) {
-          finalizeNewEntity( entity );
-          // this ensures to update the near/far planes with the exact bounding box of the new entity.
-          updateCameraNearFarPlanes();
-        } );
-
-        connect( sceneNewEntity, &Qgs3DMapSceneEntity::pendingJobsCountChanged, this, &Qgs3DMapScene::totalPendingJobsCountChanged );
+        // also sets this scene as the entity's parent and finalizes it
+        addSceneEntity( sceneNewEntity );
+      }
+      else
+      {
+        newEntity->setParent( this );
+        finalizeNewEntity( newEntity );
       }
     }
   }
 
-  if ( needsSceneUpdate )
-    onCameraChanged(); // needed for chunked entities
-
   connect( layer, &QgsMapLayer::request3DUpdate, this, &Qgs3DMapScene::onLayerRenderer3DChanged );
 
   if ( layer->type() == Qgis::LayerType::Vector )
@@ -722,11 +793,14 @@ void Qgs3DMapScene::removeLayerEntity( QgsMapLayer *layer )
 
   if ( Qgs3DMapSceneEntity *sceneEntity = qobject_cast<Qgs3DMapSceneEntity *>( entity ) )
   {
-    mSceneEntities.removeOne( sceneEntity );
+    // also schedules the entity for deletion
+    removeSceneEntity( sceneEntity );
+  }
+  else
+  {
+    if ( entity )
+      entity->deleteLater();
   }
-
-  if ( entity )
-    entity->deleteLater();
 
   disconnect( layer, &QgsMapLayer::request3DUpdate, this, &Qgs3DMapScene::onLayerRenderer3DChanged );
 
@@ -788,7 +862,7 @@ void Qgs3DMapScene::finalizeNewEntity( Qt3DCore::QEntity *newEntity )
 
   // Finalize adding the 3D transparent objects by adding the layer components to the entities
   QgsFrameGraph *frameGraph = mEngine->frameGraph();
-  Qt3DRender::QLayer *transparentLayer = frameGraph->transparentObjectLayer();
+  Qt3DRender::QLayer *transparentLayer = frameGraph->forwardRenderView().transparentObjectLayer();
   const QList<Qt3DRender::QMaterial *> childMaterials = newEntity->findChildren<Qt3DRender::QMaterial *>();
   for ( Qt3DRender::QMaterial *material : childMaterials )
   {
@@ -937,50 +1011,22 @@ void Qgs3DMapScene::onSkyboxSettingsChanged()
 
 void Qgs3DMapScene::onShadowSettingsChanged()
 {
-  QgsFrameGraph *frameGraph = mEngine->frameGraph();
-
-  const QList<QgsLightSource *> lightSources = mMap.lightSources();
-  QList<QgsDirectionalLightSettings *> directionalLightSources;
-  for ( QgsLightSource *source : lightSources )
-  {
-    if ( source->type() == Qgis::LightSourceType::Directional )
-    {
-      directionalLightSources << qgis::down_cast<QgsDirectionalLightSettings *>( source );
-    }
-  }
-
-  QgsShadowSettings shadowSettings = mMap.shadowSettings();
-  int selectedLight = shadowSettings.selectedDirectionalLight();
-  if ( shadowSettings.renderShadows() && selectedLight >= 0 && selectedLight < directionalLightSources.count() )
-  {
-    frameGraph->setShadowRenderingEnabled( true );
-    frameGraph->setShadowBias( shadowSettings.shadowBias() );
-    frameGraph->setShadowMapResolution( shadowSettings.shadowMapResolution() );
-    QgsDirectionalLightSettings light = *directionalLightSources.at( selectedLight );
-    frameGraph->setupDirectionalLight( light, shadowSettings.maximumShadowRenderingDistance() );
-  }
-  else
-    frameGraph->setShadowRenderingEnabled( false );
+  mEngine->frameGraph()->updateShadowSettings( mMap.shadowSettings(), mMap.lightSources() );
 }
 
 void Qgs3DMapScene::onAmbientOcclusionSettingsChanged()
 {
-  QgsFrameGraph *frameGraph = mEngine->frameGraph();
-  QgsAmbientOcclusionSettings ambientOcclusionSettings = mMap.ambientOcclusionSettings();
-  frameGraph->setAmbientOcclusionEnabled( ambientOcclusionSettings.isEnabled() );
-  frameGraph->setAmbientOcclusionRadius( ambientOcclusionSettings.radius() );
-  frameGraph->setAmbientOcclusionIntensity( ambientOcclusionSettings.intensity() );
-  frameGraph->setAmbientOcclusionThreshold( ambientOcclusionSettings.threshold() );
+  mEngine->frameGraph()->updateAmbientOcclusionSettings( mMap.ambientOcclusionSettings() );
 }
 
 void Qgs3DMapScene::onDebugShadowMapSettingsChanged()
 {
-  mEngine->frameGraph()->setupShadowMapDebugging( mMap.debugShadowMapEnabled(), mMap.debugShadowMapCorner(), mMap.debugShadowMapSize() );
+  mEngine->frameGraph()->updateDebugShadowMapSettings( mMap );
 }
 
 void Qgs3DMapScene::onDebugDepthMapSettingsChanged()
 {
-  mEngine->frameGraph()->setupDepthMapDebugging( mMap.debugDepthMapEnabled(), mMap.debugDepthMapCorner(), mMap.debugDepthMapSize() );
+  mEngine->frameGraph()->updateDebugDepthMapSettings( mMap );
 }
 
 void Qgs3DMapScene::onDebugOverlayEnabledChanged()
@@ -991,10 +1037,7 @@ void Qgs3DMapScene::onDebugOverlayEnabledChanged()
 
 void Qgs3DMapScene::onEyeDomeShadingSettingsChanged()
 {
-  bool edlEnabled = mMap.eyeDomeLightingEnabled();
-  double edlStrength = mMap.eyeDomeLightingStrength();
-  double edlDistance = mMap.eyeDomeLightingDistance();
-  mEngine->frameGraph()->setupEyeDomeLighting( edlEnabled, edlStrength, edlDistance );
+  mEngine->frameGraph()->updateEyeDomeSettings( mMap );
 }
 
 void Qgs3DMapScene::onCameraMovementSpeedChanged()
@@ -1082,11 +1125,11 @@ QgsRectangle Qgs3DMapScene::sceneExtent() const
   return mMap.extent();
 }
 
-QgsDoubleRange Qgs3DMapScene::elevationRange() const
+QgsDoubleRange Qgs3DMapScene::elevationRange( const bool ignoreTerrain ) const
 {
   double zMin = std::numeric_limits<double>::max();
   double zMax = std::numeric_limits<double>::lowest();
-  if ( mMap.terrainRenderingEnabled() && mTerrain )
+  if ( mMap.terrainRenderingEnabled() && mTerrain && !ignoreTerrain )
   {
     const QgsBox3D box3D = mTerrain->rootNode()->box3D();
     zMin = std::min( zMin, box3D.zMinimum() );
@@ -1197,7 +1240,33 @@ void Qgs3DMapScene::onOriginChanged()
     transform->setOrigin( mMap.origin() );
   }
 
+  const QList<QgsGeoTransform *> rubberBandGeoTransforms = mEngine->frameGraph()->rubberBandsRootEntity()->findChildren<QgsGeoTransform *>();
+  for ( QgsGeoTransform *transform : rubberBandGeoTransforms )
+  {
+    transform->setOrigin( mMap.origin() );
+  }
+
+  const QgsVector3D oldOrigin = mCameraController->origin();
   mCameraController->setOrigin( mMap.origin() );
+
+  if ( !mClipPlanesEquations.isEmpty() )
+  {
+    // how the math works - for a plane defined as (a,b,c,d), only "d" changes when
+    // moving the origin - the plane normal vector (a,b,c) stays the same.
+    // - line equation for old shift: a * (x - x0) + b * (y - y0) + c * (z - z0) + d0 = 0
+    // - line equation for new shift: a * (x - x1) + b * (y - y1) + c * (z - z1) + d1 = 0
+    // - we solve for d1:
+    //     d1 = a * (x1 - x0) + b * (y1 - y0) + c * (z1 - z0) + d0
+
+    QList<QVector4D> newPlanes;
+    QgsVector3D originShift = mMap.origin() - oldOrigin;
+    for ( QVector4D plane : std::as_const( mClipPlanesEquations ) )
+    {
+      plane.setW( originShift.x() * plane.x() + originShift.y() * plane.y() + originShift.z() * plane.z() + plane.w() );
+      newPlanes.append( plane );
+    }
+    enableClipping( newPlanes );
+  }
 }
 
 void Qgs3DMapScene::handleClippingOnEntity( QEntity *entity ) const
@@ -1241,6 +1310,10 @@ void Qgs3DMapScene::handleClippingOnAllEntities() const
   {
     handleClippingOnEntity( mTerrain );
   }
+  if ( mGlobe )
+  {
+    handleClippingOnEntity( mGlobe );
+  }
 }
 
 void Qgs3DMapScene::enableClipping( const QList<QVector4D> &clipPlaneEquations )
@@ -1252,8 +1325,7 @@ void Qgs3DMapScene::enableClipping( const QList<QVector4D> &clipPlaneEquations )
   mClipPlanesEquations = clipPlaneEquations.mid( 0, mMaxClipPlanes );
 
   // enable the clip planes on the framegraph
-  QgsFrameGraph *frameGraph = mEngine->frameGraph();
-  frameGraph->addClipPlanes( clipPlaneEquations.size() );
+  mEngine->frameGraph()->addClipPlanes( clipPlaneEquations.size() );
 
   // Enable the clip planes for the material of each entity.
   handleClippingOnAllEntities();
@@ -1264,8 +1336,7 @@ void Qgs3DMapScene::disableClipping()
   mClipPlanesEquations.clear();
 
   // disable the clip planes on the framegraph
-  QgsFrameGraph *frameGraph = mEngine->frameGraph();
-  frameGraph->removeClipPlanes();
+  mEngine->frameGraph()->removeClipPlanes();
 
   // Disable the clip planes for the material of each entity.
   handleClippingOnAllEntities();
