@@ -234,7 +234,7 @@ QgsMemoryProvider::QgsMemoryProvider( const QString &uri, const ProviderOptions
 
 QgsMemoryProvider::~QgsMemoryProvider()
 {
-  delete mSpatialIndex;
+
 }
 
 QString QgsMemoryProvider::providerKey()
@@ -682,7 +682,7 @@ bool QgsMemoryProvider::changeAttributeValues( const QgsChangedAttributesMap &at
         continue;
 
       QVariant attrValue = it2.value();
-      if ( attrValue.userType() == qMetaTypeId< QgsUnsetAttributeValue >() )
+      if ( QgsVariantUtils::isUnsetAttributeValue( attrValue ) )
         continue;
 
       // Check attribute conversion
@@ -789,7 +789,7 @@ bool QgsMemoryProvider::createSpatialIndex()
 {
   if ( !mSpatialIndex )
   {
-    mSpatialIndex = new QgsSpatialIndex();
+    mSpatialIndex = std::make_unique<QgsSpatialIndex>();
 
     // add existing features to index
     for ( QgsFeatureMap::iterator it = mFeatures.begin(); it != mFeatures.end(); ++it )
