@@ -17,6 +17,7 @@
 
 
 #include "qgsprocessingmodelerparameterwidget.h"
+#include "moc_qgsprocessingmodelerparameterwidget.cpp"
 #include "qgsprocessingparameters.h"
 #include "qgsexpressionlineedit.h"
 #include "qgsprocessingguiregistry.h"
@@ -26,6 +27,8 @@
 #include "qgsguiutils.h"
 #include "qgsexpressioncontext.h"
 #include "qgsapplication.h"
+#include "qgsprocessingregistry.h"
+#include "qgsprocessingparametertype.h"
 #include "qgsfilterlineedit.h"
 #include <QHBoxLayout>
 #include <QToolButton>
@@ -34,10 +37,7 @@
 #include <QLabel>
 #include <QComboBox>
 
-QgsProcessingModelerParameterWidget::QgsProcessingModelerParameterWidget( QgsProcessingModelAlgorithm *model,
-    const QString &childId,
-    const QgsProcessingParameterDefinition *parameter, QgsProcessingContext &context,
-    QWidget *parent )
+QgsProcessingModelerParameterWidget::QgsProcessingModelerParameterWidget( QgsProcessingModelAlgorithm *model, const QString &childId, const QgsProcessingParameterDefinition *parameter, QgsProcessingContext &context, QWidget *parent )
   : QWidget( parent )
   , mModel( model )
   , mChildId( childId )
@@ -55,7 +55,7 @@ QgsProcessingModelerParameterWidget::QgsProcessingModelerParameterWidget( QgsPro
     const QVariantList acceptedSourcesMetadata = mParameterDefinition->metadata().value( QStringLiteral( "model_widget" ) ).toMap().value( QStringLiteral( "accepted_sources" ) ).toList();
     for ( const QVariant &acceptedSource : acceptedSourcesMetadata )
     {
-      mLimitedSources.append( static_cast< Qgis::ProcessingModelChildParameterSource >( acceptedSource.toInt() ) );
+      mLimitedSources.append( static_cast<Qgis::ProcessingModelChildParameterSource>( acceptedSource.toInt() ) );
     }
   }
 
@@ -63,7 +63,7 @@ QgsProcessingModelerParameterWidget::QgsProcessingModelerParameterWidget( QgsPro
   mSourceButton->setFocusPolicy( Qt::StrongFocus );
 
   // button width is 1.25 * icon size, height 1.1 * icon size. But we round to ensure even pixel sizes for equal margins
-  mSourceButton->setFixedSize( 2 * static_cast< int >( 1.25 * iconSize / 2.0 ), 2 * static_cast< int >( iconSize * 1.1 / 2.0 ) );
+  mSourceButton->setFixedSize( 2 * static_cast<int>( 1.25 * iconSize / 2.0 ), 2 * static_cast<int>( iconSize * 1.1 / 2.0 ) );
   mSourceButton->setIconSize( QSize( iconSize, iconSize ) );
   mSourceButton->setPopupMode( QToolButton::InstantPopup );
 
@@ -76,7 +76,7 @@ QgsProcessingModelerParameterWidget::QgsProcessingModelerParameterWidget( QgsPro
 
   mStackedWidget = new QStackedWidget();
 
-  mStaticWidgetWrapper.reset( QgsGui::processingGuiRegistry()->createParameterWidgetWrapper( mParameterDefinition, QgsProcessingGui::Modeler ) );
+  mStaticWidgetWrapper.reset( QgsGui::processingGuiRegistry()->createParameterWidgetWrapper( mParameterDefinition, Qgis::ProcessingMode::Modeler ) );
   if ( mStaticWidgetWrapper )
   {
     QWidget *widget = mStaticWidgetWrapper->createWrappedWidget( context );
@@ -218,10 +218,9 @@ QVariant QgsProcessingModelerParameterWidget::value() const
       if ( v.userType() == QMetaType::Type::QVariantList )
       {
         const QVariantList vList = v.toList();
-        if ( std::all_of( vList.begin(), vList.end(), []( const QVariant & val )
-      {
-        return val.userType() == qMetaTypeId<QgsProcessingModelChildParameterSource>();
-        } ) )
+        if ( std::all_of( vList.begin(), vList.end(), []( const QVariant &val ) {
+               return val.userType() == qMetaTypeId<QgsProcessingModelChildParameterSource>();
+             } ) )
         {
           return v;
         }
@@ -242,7 +241,7 @@ QVariant QgsProcessingModelerParameterWidget::value() const
     }
 
     case ModelOutput:
-      return mModelOutputName ? ( mModelOutputName->text().trimmed().isEmpty() ? QVariant() :  mModelOutputName->text() ) : QVariant();
+      return mModelOutputName ? ( mModelOutputName->text().trimmed().isEmpty() ? QVariant() : mModelOutputName->text() ) : QVariant();
   }
 
   return QVariant::fromValue( QgsProcessingModelChildParameterSource() );
@@ -334,13 +333,13 @@ void QgsProcessingModelerParameterWidget::sourceMenuAboutToShow()
 
 void QgsProcessingModelerParameterWidget::sourceMenuActionTriggered( QAction *action )
 {
-  const Qgis::ProcessingModelChildParameterSource sourceType = action->data().value< Qgis::ProcessingModelChildParameterSource >();
+  const Qgis::ProcessingModelChildParameterSource sourceType = action->data().value<Qgis::ProcessingModelChildParameterSource>();
   setSourceType( sourceType );
 }
 
 QgsProcessingModelerParameterWidget::SourceType QgsProcessingModelerParameterWidget::currentSourceType() const
 {
-  return static_cast< SourceType >( mStackedWidget->currentIndex() );
+  return static_cast<SourceType>( mStackedWidget->currentIndex() );
 }
 
 void QgsProcessingModelerParameterWidget::setSourceType( Qgis::ProcessingModelChildParameterSource type )
@@ -355,21 +354,21 @@ void QgsProcessingModelerParameterWidget::setSourceType( Qgis::ProcessingModelCh
   switch ( type )
   {
     case Qgis::ProcessingModelChildParameterSource::StaticValue:
-      mStackedWidget->setCurrentIndex( static_cast< int >( StaticValue ) );
+      mStackedWidget->setCurrentIndex( static_cast<int>( StaticValue ) );
       mSourceButton->setIcon( QgsApplication::getThemeIcon( QStringLiteral( "mIconFieldInteger.svg" ) ) );
       mSourceButton->setToolTip( tr( "Value" ) );
       break;
 
     case Qgis::ProcessingModelChildParameterSource::Expression:
       mSourceButton->setIcon( QgsApplication::getThemeIcon( QStringLiteral( "mIconExpression.svg" ) ) );
-      mStackedWidget->setCurrentIndex( static_cast< int >( Expression ) );
+      mStackedWidget->setCurrentIndex( static_cast<int>( Expression ) );
       mSourceButton->setToolTip( tr( "Pre-calculated Value" ) );
       break;
 
     case Qgis::ProcessingModelChildParameterSource::ModelParameter:
     {
       mSourceButton->setIcon( QgsApplication::getThemeIcon( QStringLiteral( "processingModel.svg" ) ) );
-      mStackedWidget->setCurrentIndex( static_cast< int >( ModelParameter ) );
+      mStackedWidget->setCurrentIndex( static_cast<int>( ModelParameter ) );
       mSourceButton->setToolTip( tr( "Model Input" ) );
       break;
     }
@@ -377,7 +376,7 @@ void QgsProcessingModelerParameterWidget::setSourceType( Qgis::ProcessingModelCh
     case Qgis::ProcessingModelChildParameterSource::ChildOutput:
     {
       mSourceButton->setIcon( QgsApplication::getThemeIcon( QStringLiteral( "processingAlgorithm.svg" ) ) );
-      mStackedWidget->setCurrentIndex( static_cast< int >( ChildOutput ) );
+      mStackedWidget->setCurrentIndex( static_cast<int>( ChildOutput ) );
       mSourceButton->setToolTip( tr( "Algorithm Output" ) );
       break;
     }
@@ -385,7 +384,7 @@ void QgsProcessingModelerParameterWidget::setSourceType( Qgis::ProcessingModelCh
     case Qgis::ProcessingModelChildParameterSource::ModelOutput:
     {
       mSourceButton->setIcon( QgsApplication::getThemeIcon( QStringLiteral( "mIconModelOutput.svg" ) ) );
-      mStackedWidget->setCurrentIndex( static_cast< int >( ModelOutput ) );
+      mStackedWidget->setCurrentIndex( static_cast<int>( ModelOutput ) );
       mSourceButton->setToolTip( tr( "Model Output" ) );
       break;
     }
@@ -415,8 +414,7 @@ void QgsProcessingModelerParameterWidget::updateUi()
 
 void QgsProcessingModelerParameterWidget::populateSources( const QStringList &compatibleParameterTypes, const QStringList &compatibleOutputTypes, const QList<int> &compatibleDataTypes )
 {
-  const QList< QgsProcessingModelChildParameterSource > sources = mModel->availableSourcesForChild( mChildId,
-      compatibleParameterTypes, compatibleOutputTypes, compatibleDataTypes );
+  QgsProcessingModelChildParameterSources sources = mModel->availableSourcesForChild( mChildId, compatibleParameterTypes, compatibleOutputTypes, compatibleDataTypes );
 
   for ( const QgsProcessingModelChildParameterSource &source : sources )
   {
@@ -447,7 +445,6 @@ void QgsProcessingModelerParameterWidget::populateSources( const QStringList &co
       case Qgis::ProcessingModelChildParameterSource::ModelOutput:
         break;
     }
-
   }
 }
 
