@@ -32,12 +32,13 @@ class DummyMaterialSettings : public QgsAbstractMaterialSettings
     static QgsAbstractMaterialSettings *create() { return new DummyMaterialSettings(); }
     DummyMaterialSettings *clone() const override { return new DummyMaterialSettings(); }
     static bool supportsTechnique( QgsMaterialSettingsRenderingTechnique ) { return true; }
-    void readXml( const QDomElement &, const QgsReadWriteContext & ) override { }
+    void readXml( const QDomElement &, const QgsReadWriteContext & ) override {}
     void writeXml( QDomElement &, const QgsReadWriteContext & ) const override {}
     void addParametersToEffect( Qt3DRender::QEffect *, const QgsMaterialContext & ) const override {}
     QgsMaterial *toMaterial( QgsMaterialSettingsRenderingTechnique, const QgsMaterialContext & ) const override { return nullptr; }
     QMap<QString, QString> toExportParameters() const override { return QMap<QString, QString>(); }
-    QByteArray dataDefinedVertexColorsAsByte( const QgsExpressionContext & ) const override {return QByteArray();}
+    QByteArray dataDefinedVertexColorsAsByte( const QgsExpressionContext & ) const override { return QByteArray(); }
+    bool equals( const QgsAbstractMaterialSettings * ) const override { return true; }
 };
 
 class TestQgsMaterialRegistry : public QgsTest
@@ -62,7 +63,6 @@ class TestQgsMaterialRegistry : public QgsTest
     void createMaterial();
 
   private:
-
 };
 
 void TestQgsMaterialRegistry::initTestCase()
@@ -78,23 +78,20 @@ void TestQgsMaterialRegistry::cleanupTestCase()
 
 void TestQgsMaterialRegistry::init()
 {
-
 }
 
 void TestQgsMaterialRegistry::cleanup()
 {
-
 }
 
 void TestQgsMaterialRegistry::metadata()
 {
-  QgsMaterialSettingsMetadata metadata = QgsMaterialSettingsMetadata( QStringLiteral( "name" ), QStringLiteral( "display name" ),
-                                         DummyMaterialSettings::create, DummyMaterialSettings::supportsTechnique );
+  QgsMaterialSettingsMetadata metadata = QgsMaterialSettingsMetadata( QStringLiteral( "name" ), QStringLiteral( "display name" ), DummyMaterialSettings::create, DummyMaterialSettings::supportsTechnique );
   QCOMPARE( metadata.type(), QString( "name" ) );
   QCOMPARE( metadata.visibleName(), QString( "display name" ) );
 
   //test creating material settings from metadata
-  const std::unique_ptr< QgsAbstractMaterialSettings > material( metadata.create() );
+  const std::unique_ptr<QgsAbstractMaterialSettings> material( metadata.create() );
   QVERIFY( material );
   DummyMaterialSettings *dummyMaterial = dynamic_cast<DummyMaterialSettings *>( material.get() );
   QVERIFY( dummyMaterial );
@@ -123,13 +120,11 @@ void TestQgsMaterialRegistry::addMaterialSettings()
   QgsMaterialRegistry *registry = Qgs3D::materialRegistry();
   const int previousCount = registry->materialSettingsTypes().length();
 
-  registry->addMaterialSettingsType( new QgsMaterialSettingsMetadata( QStringLiteral( "Dummy" ), QStringLiteral( "Dummy material" ),
-                                     DummyMaterialSettings::create, DummyMaterialSettings::supportsTechnique ) );
+  registry->addMaterialSettingsType( new QgsMaterialSettingsMetadata( QStringLiteral( "Dummy" ), QStringLiteral( "Dummy material" ), DummyMaterialSettings::create, DummyMaterialSettings::supportsTechnique ) );
   QCOMPARE( registry->materialSettingsTypes().length(), previousCount + 1 );
   //try adding again, should have no effect
-  QgsMaterialSettingsMetadata *dupe = new QgsMaterialSettingsMetadata( QStringLiteral( "Dummy" ), QStringLiteral( "Dummy material" ),
-      DummyMaterialSettings::create, DummyMaterialSettings::supportsTechnique );
-  QVERIFY( ! registry->addMaterialSettingsType( dupe ) );
+  QgsMaterialSettingsMetadata *dupe = new QgsMaterialSettingsMetadata( QStringLiteral( "Dummy" ), QStringLiteral( "Dummy material" ), DummyMaterialSettings::create, DummyMaterialSettings::supportsTechnique );
+  QVERIFY( !registry->addMaterialSettingsType( dupe ) );
   QCOMPARE( registry->materialSettingsTypes().length(), previousCount + 1 );
   delete dupe;
 
@@ -156,7 +151,7 @@ void TestQgsMaterialRegistry::fetchTypes()
 void TestQgsMaterialRegistry::createMaterial()
 {
   QgsMaterialRegistry *registry = Qgs3D::materialRegistry();
-  std::unique_ptr< QgsAbstractMaterialSettings > material( registry->createMaterialSettings( QStringLiteral( "Dummy" ) ) );
+  std::unique_ptr<QgsAbstractMaterialSettings> material( registry->createMaterialSettings( QStringLiteral( "Dummy" ) ) );
 
   QVERIFY( material.get() );
   DummyMaterialSettings *dummySymbol = dynamic_cast<DummyMaterialSettings *>( material.get() );
