@@ -28,6 +28,7 @@ import urllib.request
 import osgeo.gdal  # NOQA
 
 from qgis.core import (
+    Qgis,
     QgsCoordinateReferenceSystem,
     QgsFeature,
     QgsField,
@@ -37,9 +38,16 @@ from qgis.core import (
     QgsMemoryProviderUtils,
     QgsProject,
     QgsWkbTypes,
+    QgsPointXY,
+    QgsAttributeEditorField,
+)
+from qgis.PyQt.QtCore import QVariant, QUrl
+from qgis.server import (
+    QgsBufferServerRequest,
+    QgsBufferServerResponse,
+    QgsServer,
+    QgsServerRequest,
 )
-from qgis.PyQt.QtCore import QVariant
-from qgis.server import QgsBufferServerRequest, QgsBufferServerResponse
 from qgis.testing import unittest, QgisTestCase
 from test_qgsserver_wms import TestQgsServerWMSTestBase
 
@@ -968,7 +976,9 @@ class TestQgsServerWMSGetFeatureInfo(TestQgsServerWMSTestBase):
         j_body = json.loads(bytes(res.body()).decode())
         self.assertEqual(len(j_body["features"]), 2)
 
-        vl1.setFlags(vl1.flags() & ~QgsMapLayer.LayerFlag.Identifiable)
+        vl1.setFlags(
+            QgsMapLayer.LayerFlag(vl1.flags() & ~QgsMapLayer.LayerFlag.Identifiable)
+        )
 
         req = QgsBufferServerRequest(
             "?" + "&".join([f"{k}={v}" for k, v in req_params.items()])
@@ -1229,9 +1239,11 @@ class TestQgsServerWMSGetFeatureInfo(TestQgsServerWMSTestBase):
             + urllib.parse.quote(":\"XXXXXXXXXNAMEXXXXXXX\" = 'two'"),
         )
 
-        self.assertEqual(
-            response_body.decode("utf8"),
-            '<?xml version="1.0" encoding="UTF-8"?>\n<ServiceExceptionReport xmlns="http://www.opengis.net/ogc" version="1.3.0">\n <ServiceException code="InvalidParameterValue">Filter not valid for layer testlayer èé: check the filter syntax and the field names.</ServiceException>\n</ServiceExceptionReport>\n',
+        self.assertTrue(
+            response_body.decode("utf8").find(
+                '<ServiceException code="InvalidParameterValue">Filter not valid for layer testlayer èé: check the filter syntax and the field names.</ServiceException>\n</ServiceExceptionReport>\n'
+            )
+            >= 0
         )
 
     def testGetFeatureInfoFilterAllowedExtraTokens(self):
@@ -1357,6 +1369,197 @@ class TestQgsServerWMSGetFeatureInfo(TestQgsServerWMSTestBase):
             "test_project_values.qgz",
         )
 
+    def test_getfeatureinfo_sub_px_tolerance(self):
+
+        # create a memory layer with points
+        fields = QgsFields()
+        fields.append(QgsField("id", QVariant.Int))
+        fields.append(QgsField("name", QVariant.String))
+        layer = QgsMemoryProviderUtils.createMemoryLayer(
+            "points",
+            fields,
+            QgsWkbTypes.Point,
+            QgsCoordinateReferenceSystem("EPSG:4326"),
+        )
+
+        provider = layer.dataProvider()
+        self.assertTrue(layer.isValid())
+
+        # add some features
+        f = QgsFeature(fields)
+        f.setAttribute("id", 1)
+        f.setAttribute("name", "point1")
+        f.setGeometry(QgsGeometry.fromPointXY(QgsPointXY(0, 0)))
+        provider.addFeature(f)
+
+        f = QgsFeature(fields)
+        f.setAttribute("id", 2)
+        f.setAttribute("name", "point2")
+        f.setGeometry(QgsGeometry.fromPointXY(QgsPointXY(1, 1)))
+        provider.addFeature(f)
+
+        f = QgsFeature(fields)
+        f.setAttribute("id", 3)
+        f.setAttribute("name", "point3")
+        f.setGeometry(QgsGeometry.fromPointXY(QgsPointXY(-1, -1)))
+        provider.addFeature(f)
+
+        project = QgsProject()
+        project.addMapLayer(layer)
+
+        # set up the WMS server
+        server = QgsServer()
+        request = QgsServerRequest()
+        # Choose a small extent to trigger the sub-pixel tolerance adjustment
+        w = 10
+        w2 = int(w / 2)
+        request.setUrl(
+            QUrl(
+                f"?SERVICE=WMS&REQUEST=GetFeatureInfo&LAYERS=points&QUERY_LAYERS=points&INFO_FORMAT=application/json&FEATURE_COUNT=1&WIDTH={w}&HEIGHT={w}&CRS=EPSG:4326&STYLES=&BBOX=-1,-1,1,1&X={w2}&Y={w2}&VERSION=1.3.0"
+            )
+        )
+        response = QgsBufferServerResponse()
+
+        server.handleRequest(request, response, project)
+        body = response.body().data().decode("utf8").replace("\n", "")
+        self.assertEqual(len(json.loads(body)["features"]), 1)
+
+    def test_getfeatureinfo_form_config(self):
+        """Test issue GH #59335 : getfeatureinfo with form config drag and drop settings"""
+
+        # create a memory layer with points
+        fields = QgsFields()
+        fields.append(QgsField("id_xyz", QVariant.Int))
+        fields.append(QgsField("name_xyz", QVariant.String))
+        layer = QgsMemoryProviderUtils.createMemoryLayer(
+            "points",
+            fields,
+            QgsWkbTypes.Point,
+            QgsCoordinateReferenceSystem("EPSG:4326"),
+        )
+
+        provider = layer.dataProvider()
+        self.assertTrue(layer.isValid())
+
+        # add some features
+        f = QgsFeature(fields)
+        f.setAttribute("id_xyz", 1)
+        f.setAttribute("name_xyz", "point1")
+        f.setGeometry(QgsGeometry.fromPointXY(QgsPointXY(0, 0)))
+        provider.addFeature(f)
+
+        f = QgsFeature(fields)
+        f.setAttribute("id_xyz", 2)
+        f.setAttribute("name_xyz", "point2")
+        f.setGeometry(QgsGeometry.fromPointXY(QgsPointXY(1, 1)))
+        provider.addFeature(f)
+
+        f = QgsFeature(fields)
+        f.setAttribute("id_xyz", 3)
+        f.setAttribute("name_xyz", "point3")
+        f.setGeometry(QgsGeometry.fromPointXY(QgsPointXY(-1, -1)))
+        provider.addFeature(f)
+
+        project = QgsProject()
+        project.addMapLayer(layer)
+        project.writeEntry("WMSFeatureInfoUseAttributeFormSettings", "/", "true")
+
+        # Set the form settings
+        editFormConfig = layer.editFormConfig()
+        editFormConfig.clearTabs()
+        editFormConfig.addTab(
+            QgsAttributeEditorField(
+                "id_xyz", 0, editFormConfig.invisibleRootContainer()
+            )
+        )
+        editFormConfig.setLayout(Qgis.AttributeFormLayout.DragAndDrop)
+        layer.setEditFormConfig(editFormConfig)
+
+        # set up the WMS server
+        server = QgsServer()
+        request = QgsServerRequest()
+        w = 10
+        w2 = int(w / 2)
+
+        for info_format in [
+            "text/plain",
+            "application/json",
+            "text/xml",
+            "application/vnd.ogc.gml",
+        ]:
+
+            request.setUrl(
+                QUrl(
+                    f"?SERVICE=WMS&REQUEST=GetFeatureInfo&LAYERS=points&QUERY_LAYERS=points&INFO_FORMAT={info_format}&FEATURE_COUNT=1&WIDTH={w}&HEIGHT={w}&CRS=EPSG:4326&STYLES=&BBOX=-1,-1,1,1&X={w2}&Y={w2}&VERSION=1.3.0"
+                )
+            )
+            response = QgsBufferServerResponse()
+
+            server.handleRequest(request, response, project)
+            body = response.body().data().decode("utf8").replace("\n", " ")
+            self.assertNotIn("name_xyz", body)
+            self.assertIn("id_xyz", body)
+
+    def test_getfeatureinfo_display_name(self):
+        """Test issue GH #59353"""
+
+        # create a memory layer with points
+        fields = QgsFields()
+        fields.append(QgsField("id_xyz", QVariant.Int))
+        fields.append(QgsField("name_xyz", QVariant.String))
+        layer = QgsMemoryProviderUtils.createMemoryLayer(
+            "points",
+            fields,
+            QgsWkbTypes.Point,
+            QgsCoordinateReferenceSystem("EPSG:4326"),
+        )
+        layer.setDisplayExpression("name_xyz || ' (' || id_xyz || ')'")
+
+        provider = layer.dataProvider()
+        self.assertTrue(layer.isValid())
+
+        # add some features
+        f = QgsFeature(fields)
+        f.setAttribute("id_xyz", 1)
+        f.setAttribute("name_xyz", "point1")
+        f.setGeometry(QgsGeometry.fromPointXY(QgsPointXY(0, 0)))
+        provider.addFeature(f)
+
+        f = QgsFeature(fields)
+        f.setAttribute("id_xyz", 2)
+        f.setAttribute("name_xyz", "point2")
+        f.setGeometry(QgsGeometry.fromPointXY(QgsPointXY(1, 1)))
+        provider.addFeature(f)
+
+        f = QgsFeature(fields)
+        f.setAttribute("id_xyz", 3)
+        f.setAttribute("name_xyz", "point3")
+        f.setGeometry(QgsGeometry.fromPointXY(QgsPointXY(-1, -1)))
+        provider.addFeature(f)
+
+        project = QgsProject()
+        project.addMapLayer(layer)
+
+        # set up the WMS server
+        server = QgsServer()
+        request = QgsServerRequest()
+        w = 10
+        w2 = int(w / 2)
+
+        # Note: not implemented for , 'application/vnd.ogc.gml'
+        for info_format in ["text/plain", "application/json", "text/xml"]:
+
+            request.setUrl(
+                QUrl(
+                    f"?SERVICE=WMS&REQUEST=GetFeatureInfo&LAYERS=points&QUERY_LAYERS=points&INFO_FORMAT={info_format}&FEATURE_COUNT=1&WIDTH={w}&HEIGHT={w}&CRS=EPSG:4326&STYLES=&WITH_DISPLAY_NAME=true&BBOX=-1,-1,1,1&X={w2}&Y={w2}&VERSION=1.3.0"
+                )
+            )
+            response = QgsBufferServerResponse()
+
+            server.handleRequest(request, response, project)
+            body = response.body().data().decode("utf8").replace("\n", " ")
+            self.assertIn("point1 (1)", body)
+
 
 if __name__ == "__main__":
     unittest.main()
